"""Constants for Zodiac."""
DOMAIN = "zodiac"

# Signs
SIGN_ARIES = "aries"
SIGN_TAURUS = "taurus"
SIGN_GEMINI = "gemini"
SIGN_CANCER = "cancer"
SIGN_LEO = "leo"
SIGN_VIRGO = "virgo"
SIGN_LIBRA = "libra"
SIGN_SCORPIO = "scorpio"
SIGN_SAGITTARIUS = "sagittarius"
SIGN_CAPRICORN = "capricorn"
SIGN_AQUARIUS = "aquarius"
SIGN_PISCES = "pisces"

# Elements
ELEMENT_FIRE = "fire"
ELEMENT_AIR = "air"
ELEMENT_EARTH = "earth"
ELEMENT_WATER = "water"

# Modality
MODALITY_CARDINAL = "cardinal"
MODALITY_FIXED = "fixed"
MODALITY_MUTABLE = "mutable"

# Attributes
ATTR_ELEMENT = "element"
ATTR_MODALITY = "modality"
