"""Constants for the Tuya integration."""

CONF_COUNTRYCODE = "country_code"

DOMAIN = "tuya"

TUYA_DATA = "tuya_data"
TUYA_DISCOVERY_NEW = "tuya_discovery_new_{}"

TUYA_PLATFORMS = {
    "tuya": "Tuya",
    "smart_life": "Smart Life",
    "jinvoo_smart": "Jinvoo Smart",
}
